"""Main module of Micro-CI."""
# pylint: disable=unsubscriptable-object

import subprocess
import tempfile
from typing import Any, Dict, List, Optional

import yaml


def main(disable: Optional[List] = None) -> None:
    """Monolithic main function."""

    with open(".gitlab-ci.yml", "r") as file:
        try:
            config: Dict[str, Any] = yaml.safe_load(file)
        except yaml.YAMLError as exc:
            print(exc)

    stages = config["stages"]
    only = "merge_requests"

    variables = {}
    try:
        variables = config["variables"]
    except ValueError:
        pass

    warnings = []

    for stage in stages:
        if disable and stage in disable:
            continue
        jobs = []
        for name, job in config.items():
            try:
                if job["stage"] == stage:
                    jobs.append(name)
            except (TypeError, KeyError):
                pass  # ignore when job isn't a dict or doesn't contain "stage"

        for job in jobs:
            jc = config[job]
            if (
                "only" not in jc
                or only in jc["only"]
                or "refs" in jc["only"]
                and only in jc["only"]["refs"]
            ):
                try:
                    script = jc["script"]
                    with tempfile.NamedTemporaryFile() as tmp:
                        fh = open(tmp.name, "w")
                        print("#!/bin/bash", file=fh)
                        for name, value in variables.items():
                            print(f"{name}={value}")
                            print(f"{name}={value}", file=fh)
                        for line in script:
                            print(f"{line}")
                            print(f"{line}", file=fh)
                        fh.close()

                        try:
                            subprocess.run(["bash", f"{tmp.name}"], check=True)
                        except subprocess.CalledProcessError:
                            if "allow_failure" in jc:
                                af = jc["allow_failure"]
                            else:
                                af = False

                            if af:
                                warnings.append(
                                    f"Micro-CI: Job {job} in stage {stage} failed, "
                                    "but was allowed to fail."
                                )
                                print(
                                    f"Micro-CI: Job {job} in stage {stage} failed, "
                                    "but was allowed to fail."
                                )
                            else:
                                print(f"Micro-CI: Job {job} in stage {stage} failed. Exiting.")
                                exit(1)

                except KeyError:
                    pass

    print("Micro-CI: All stages done, no errors.")
    if warnings:
        for w in warnings:
            print(w)


if __name__ == "__main__":
    main()



# Copyright (c) 2020-2021, Gauss Machine Learning GmbH. All rights reserved.
# This file is part of Micro-CI, which is released under the BSD 3-Clause License.

"""Setup and metadata for the Micro-CI package."""

import pathlib

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

# For details on the metadata specification, have a look at the example or the documentation:
# https://github.com/pypa/sampleproject/blob/master/setup.py
# https://packaging.python.org/guides/distributing-packages-using-setuptools/

setup(
    name="micro_ci",
    description="Micro-CI: A minimalist runner for GitLab CI configurations.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/gauss-ml-open/micro-ci",
    author="Edgar Klenske",
    author_email="edgar.klenske@gauss-ml.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="development",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.5, <4",
    install_requires=["pyyaml"],
    extras_require={
        "dev": ["black", "flake8", "mypy", "pylint", "rope"],
        "test": ["flaky", "pytest", "pytest-cov"],
    },
    entry_points={
        "console_scripts": [
            "mci=micro_ci:main",
        ]
    },
)
